#!/usr/bin/perl -w

use strict;

use DBI;

use Text::Wrap qw(wrap);

$Text::Wrap::columns = 72;

my $conn = DBI->connect("dbi:Pg:dbname=osp_rules");

my $query = 
    $conn->prepare(
        "SELECT index, text, suggestor, phraser, start_seq, history " . 
        "FROM rules " .
        "ORDER BY index"
    );

$query->execute();

my (@columns);

my $is_first = 1;
my $last_index = -1;
print "The Rules of Open-Source Programming\n\n\n";

my @cookies = ();
my @this_cookie_rules = ();
my $this_cookie_start_idx = -1;

open VERBOSE, ">osp_rules.verbose.txt";
while (@columns = $query->fetchrow_array())
{
    my ($index, $text, $suggestor, $phraser, $start_seq, $history) = @columns;
   
    print wrap('', '', "$index. $text");
    print "\n\n";

    # Print the verbose dump
    print VERBOSE "Index: $index\n";
    print VERBOSE "Text: $text\n";
    print VERBOSE "Suggestor: $suggestor\n";
    print VERBOSE "Phraser: $phraser\n";
    print VERBOSE "Starts a new index: " . ($start_seq ? "True" : "False") . "\n";
    print VERBOSE "History: <<<\n";
    print VERBOSE wrap('', '', $history), "\n";
    print VERBOSE ">>>\n";
    print VERBOSE "\n" . ("-" x 72) . "\n";

    if ($is_first)
    {
        $this_cookie_start_idx = $index;
    }

    if (! $is_first)
    {
        if ($start_seq)
        {
            push @cookies,
                {
                    'idx' => $this_cookie_start_idx,
                    'rules' => [ @this_cookie_rules ],
                };

            @this_cookie_rules = ();
            $this_cookie_start_idx = $index;
        }
    }

    push @this_cookie_rules, $text;

    if (! $is_first)
    {
        if ($last_index == $index)
        {
            warn "Duplicate index $index!\n";
        }        
    }
    $last_index = $index;
    $is_first = 0;
}

close(VERBOSE);

push @cookies,
    {
        'idx' => $this_cookie_start_idx,
        'rules' => [ @this_cookie_rules ],
    };

sub render_cookie
{
    my $cookie = shift;

    my $index = $cookie->{'idx'};
    my @rules = @{$cookie->{'rules'}};

    if (scalar(@rules) > 1)
    {
        return 
            "Rules of Open-Source Programming:\n\n" . 
                join("\n\n", 
                        (map 
                            { wrap('','',($index+$_) . "." , $rules[$_]) } 
                            (0.. $#rules)
                        )
                    )
            ;
    }
    else
    {
        return 
            "Rule of Open-Source Programming #$index:\n\n" . wrap('','',$rules[0]);
    }
}

open FORTUNES, ">osp_rules";
print FORTUNES join("\n%\n", (map { render_cookie($_) } @cookies));
close(FORTUNES);

$conn->disconnect();
